//
// C++ Interface: gui
//
// Description: Graphical User Interface, and image computation for mulot.
//
//
//     Copyright (C) 2008  Romain Fontugne <romain.fontugne@gmail.com>
//
//     This program is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
// 
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
#ifndef GUI_H
#define GUI_H

#include <math.h>
#include <map>
#include <sstream>
#include <fstream>

#include <pthread.h>

#ifndef __MINGW32__
#include <arpa/inet.h>
#endif

#include "CImg.h"
#include "sniffer.h"
#include "protohdr.h"
#include "mulot.h"

using namespace cimg_library;
using namespace std;

struct spot {
	char fileInd;
	int  offset;
};

union header{
	IcmpHeader*	icmp;
	TcpHeader*	tcp;
	UdpHeader*	udp;
    };

struct pkt {
    struct timeval ts;
    InetAddr 	srcAddr;
    InetAddr 	destAddr;
    u_char	proto;
    int		len;		// Packet length
    header	hdr;

    pkt() {proto = 0;}
    pkt(pkt* p) { *this = *p; }
//     ~pkt(){
// 	switch (this->proto) {
// 		case IPPROTO_TCP:	delete this->hdr.tcp; break;
// 		case IPPROTO_UDP:	delete this->hdr.udp; break;
// 		case IPPROTO_ICMP:	delete this->hdr.icmp; break;
// 	}
//     }
};

typedef multimap<int,pkt> info_type;

//#include "detector.h"
void readDump(u_char* aUserPtr, const pcap_pkthdr* aHdrPtr, const u_char * aDataPtr);

/**
	@author Romain Fontugne <romain.fontugne@gmail.com>
*/

class GUI{
public:
    GUI();

    ~GUI();

    void init(bool disp=true);

    // Display functions
    void display(char loopMode = 'i', int x=-1);
    void displayRefresh(int disp);
    void infoDisplay(const char* info, int size, bool empty=true, int x=0, int y=0);
    void rulersDisplay(char mode, int disp);
    void help();

    // Zooming functions
    void zoomInX(double step, bool center=true);
    void zoomOutX(double step, bool center=true);
    void zoomInY(double step, bool center=true);
    void zoomOutY(double step, bool center=true);
    void zoomWhl();

    // Navigation
    void stepX(double step);
    void stepY(double step);
    void rtz();
    void focusCenter();
    void focusMouse();


    void wait();
    bool isClosed();
    void saveImg();
    void saveImg(int disp);

    // Y mode functions
    void updateYModeInfo(int disp=0);
    void nextYMode(int disp);
    void prevYMode(int disp);

    // glass functions
    void glassInfo(int x=-1, int y=-1);
    void glassDisplay(int glassX=-1, int glassY=-1);

    void graphIn(int x, int y);
    void graphDraw(int disp);
    void graphClean();

// Members
    Sniffer* sniff;	// object used to read the dump file
    int	eventDisplay;
    int	eventButton;
    string event;

    bool	silent;
    bool	saveRulers;

    int		glassX;
    int		glassY;
    int		glassSize;

    double 	timeCoefI;
    double 	yCoefI;
    bool 	refreshMenu;

    char colorMode;
    unsigned char *red;	// Colors
    unsigned char *blue;
    unsigned char *green;
    unsigned char *white;
    unsigned char *black;
    unsigned char *purple;
    unsigned char *orange;
    unsigned char *grey;
    unsigned char *second;
    unsigned char *last;
    int 	greyBg;
    int 	whiteBg;

    double	mouseX;
    double	mouseY[NB_YMODE];
    double	mouseYR[NB_YMODE];

    int		mainX;
    int		mainY;
    int 	menuX;
    int 	menuY;

    unsigned int  cidrDest;
    unsigned int  cidrSrc;
    unsigned int  timeRange;		// 86400 = 24hours
    double	yRange[NB_YMODE];	// if gBitSignif == 14 then yRange[0] = 16384
    double 	timeCoef0;		// Coef. so we can see timeRange seconds in the initial window
    double	timeCoef;		// Coef. for the window (x axis)
    //double 	addrCoef0;		// Coef. so we can see yRange host in the initial window
    //double	addrRangeDest;
    //double	addrRangeSrc;
    //double	portCoef0;	
    //int	portRange;
    double	yCoef0[NB_YMODE];
    double	yCoef[NB_YMODE];

    char*	yMode;
    char 	loopMode;		// image (i) or data (d)

    time_t	startTick;		// display after this tick
    time_t	firstTick;
    double	startY[NB_YMODE];
    unsigned int count;

    CImgList<unsigned char> *mainList;
    CImgList<unsigned char> *mainList2;
    CImgList<unsigned char> *menuList;
    CImgList<unsigned char> *btnList;	// Define the menu's header
    CImg<unsigned char> *mapImg;	// Define the menu picture
    CImg<unsigned char> *infoImg;	// Define the menu's bottom
    CImg<unsigned char> *rulerXImg;	// Define bottom for the main
    CImg<unsigned char> *rulerYImg;	// Define y axis for the main display
    CImg<unsigned char> *mainImg;	// Define the main picture
    CImg<unsigned int> *mainStat;	// Define the main picture

    CImg<unsigned char> detecImg[2][NB_SUB_DETEC][NB_YMODE_DETEC];	// Define the detect picture

    int nbDisp;
    CImgDisplay *mainDisp, *menuDisp;

    int cleanFreq;
    info_type *info;
    multimap<short int,unsigned int> infoDetec[2][NB_SUB_DETEC][NB_YMODE_DETEC][700];	//TODO remove hardcoded to see detector.h/cpp and gui.cpp
    vector<pkt> pktList;
    spot* seek;
    bool seekComputed;

    //boost::shared_ptr< admd::annotation_t > annotation;
    int nbAnnotation;

    //Detector detector1;
    //Detector detector2;


// Graph
  // For tcp and udp
    map<InetAddr,map<unsigned int,char> > graphSrcA;
    map<InetAddr,map<unsigned int,char> > graphDstA;
    map<unsigned int,map<unsigned int,char> > graphSrcP;
    map<unsigned int,map<unsigned int,char> > graphDstP;
  // For other protocols
    map<InetAddr,map<InetAddr,char> > graphOther;
};


#endif
